//
//  PUPlayOptionsManager.m
//  BJPlayerManagerUI_Example
//
//  Created by HuangJie on 2018/6/5.
//  Copyright © 2018年 oushizishu. All rights reserved.
//

#import <BJLiveBase/BJLiveBase+UIKit.h>
#import <BJLiveBase/BJLYYModel.h>

#import "PUPlayOptionsManager.h"

static NSString * const videoOptionsKey = @"video_options";
static NSString * const playbackOptionsKey = @"playback_options";
static NSString * const privatePrefixKey = @"private_prefix";

@implementation PUPlayOptionsManager

#pragma mark - BJPUVideoOptions 点播设置

+ (void)setVideoOptions:(BJPUVideoOptions *)videoOptions {
    id jsonObject = [videoOptions bjlyy_modelToJSONObject];
    NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
    [userDefaults setObject:jsonObject forKey:videoOptionsKey];
    [userDefaults synchronize];
}

+ (BJPUVideoOptions *)getVideoOptions {
    NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
    id jsonObject = [userDefaults objectForKey:videoOptionsKey];
    return [BJPUVideoOptions bjlyy_modelWithJSON:jsonObject] ?: [BJPUVideoOptions new];
}

#pragma mark - BJPPlaybackOptions 回放设置

+ (void)setPlaybackOptions:(BJPPlaybackOptions *)playbackOptions {
    id jsonObject = [playbackOptions bjlyy_modelToJSONObject];
    NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
    [userDefaults setObject:jsonObject forKey:playbackOptionsKey];
    [userDefaults synchronize];
}

+ (BJPPlaybackOptions *)getPlaybackOptions {
    NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
    id jsonObject = [userDefaults objectForKey:playbackOptionsKey];
    return [BJPPlaybackOptions bjlyy_modelWithJSON:jsonObject] ?: [BJPPlaybackOptions new];
}

#pragma mark - 专属域名

+ (void)setPrivatePrefix:(NSString *)privatePrefix {
    NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
    [userDefaults setObject:privatePrefix forKey:privatePrefixKey];
    [userDefaults synchronize];
}

+ (NSString *)getPrivatePrefix {
    NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
    return [userDefaults stringForKey:privatePrefixKey];
}

#pragma mark - 跑马灯

+ (void)setLamp:(BJLLamp *)lamp {
    NSDictionary *lampDic = @{@"content": lamp.content, @"size": @(lamp.fontSize), @"alpha": @(lamp.alpha), @"color": lamp.color };
    [PUPlayOptionsManager setConfiguration:lampDic key:@"lamp"];
}

+ (BJLLamp *)getLamp {
    BJPUVideoOptions *options = [PUPlayOptionsManager getVideoOptions];
    NSString *lampContent = options.userName;
    CGFloat lampSize = 14.0;
    CGFloat lampAlha = 1.0;
    NSString *lampColor = @"#FFFFFF";
    NSDictionary *dic = [PUPlayOptionsManager getConfigurationWithKey:@"lamp"];
    if (dic) {
        UIColor *color = [UIColor bjl_colorWithHexString:[dic bjl_stringForKey:@"color"]];
        if(color) {
            lampContent = [dic bjl_stringForKey:@"content"];
            lampSize = [dic bjl_floatForKey:@"size"];
            lampAlha = [dic bjl_floatForKey:@"alpha"];
            lampColor = [dic bjl_stringForKey:@"color"];
        }
    }
    return [[BJLLamp alloc] initWithContent:lampContent fontSize:lampSize color:lampColor alpha:lampAlha];
}

#pragma mark -

+ (void)setConfiguration:(NSDictionary *)configuration key:(NSString *)key {
    NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
    [userDefaults setObject:configuration forKey:key];
    [userDefaults synchronize];
}

+ (NSDictionary *)getConfigurationWithKey:(NSString *)key {
    NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
    return [userDefaults dictionaryForKey:key];
}

@end
